@@ -47,62 +47,67 @@ BEGIN {
 }
 
 
-# _Clean_Index
+# _Clean_dbh
+# _Clean_order_by
+# _Clean_data_table_name
 # _Clean_table_name
-# _Clean_sort_name
-# _Clean_sort_string
+# _Clean_sort_spec
 # _Clean_in_progress
+# _Clean_doctype
+# _Clean_indexname
 
 sub init_and_cast {
   my ( $class, %args ) = @_;
   my $self = $args{element} || die "need an element";
-  $self->{_Clean_Index} = $args{index} || die "need an index";
+  $self->{_Clean_doctype} = $args{doctype};
+  $self->{_Clean_indexname} = $args{index_name};
+  $self->{_Clean_dbh} = $args{dbh};
+  $self->{_Clean_order_by} = $args{order_by};
   $self->{_Clean_table_name} = $args{table_name} || die "need a table name";
+  $self->{_Clean_data_table_name} = $args{data_table_name};
   $self->{_Clean_bcollection_table_names} =
     $args{bcollection_table_names} || [];
-  $self->{_Clean_sort_name} = $args{sort_name};
-  $self->{_Clean_sort_string} = $args{sort_string};
+  $self->{_Clean_sort_spec} = $args{sort_spec} || '';
   bless ( $self, $class );
   return $self;
 }
 
 sub clean {
   my $self = shift();
-  my $index = $self->{_Clean_Index};
+  my $dbh = $self->{_Clean_dbh};
+  my $order_by = $self->{_Clean_order_by};
+  my $data_table_name = $self->{_Clean_data_table_name};
   my $table_name = $self->{_Clean_table_name};
-  my $dbh = $index->get_dbh();
   my $erase_where_clause = eval { $self->element('erase_where_clause')->get() };
   # don't clean if table _comma flag is set
-  if ( sql_get_table_comma_flag($index, $table_name) ) {
+  if ( sql_get_table_comma_flag($dbh, $table_name) ) {
     print "skipping clean on $table_name...";
     return;
   }
   $self->{_Clean_in_progress} = 1;
   # set table _comma flag
-  sql_set_table_comma_flag ( $index, $table_name, $clean_flag );
+  sql_set_table_comma_flag ( $dbh, $table_name, $clean_flag );
   # dbg 'cleaning', $table_name;
   # for table we care about: clear all _comma flags
-  sql_clear_all_comma_flags ( $index, $table_name );
-  # first pass clean: for sort tables removes orphan entries, for data
-  # tables removes rows matching any erase_where_clause
+  sql_clear_all_comma_flags ( $dbh, $table_name );
+  # first pass clean: for sort tables removes orphan entries, for both
+  # sort and data tables removes rows matching any erase_where_clause
   sql_set_comma_flags_for_clean_first_pass
-    ( $index,
-      $table_name,
-      $erase_where_clause,
-      $clean_flag );
-  sql_delete_where_comma_flags ( $index, $table_name, $clean_flag );
+    ( $dbh, $data_table_name, $table_name, $erase_where_clause, $clean_flag );
+  sql_delete_where_comma_flags ( $dbh, $table_name, $clean_flag );
   # second pass clean: arranges rows in order and removes rows above
   # our to_size limit
   if ( my $size_limit = $self->element('to_size')->get() ) {
     sql_set_comma_flags_for_clean_second_pass
-      ( $index,
+      ( $dbh,
         $table_name,
-        $self->element('order_by')->get() ||
-          $index->element('default_order_by')->get(),
-        $self->{_Clean_sort_name}, $self->{_Clean_sort_string},
+        $self->{_Clean_order_by},
+        $self->{_Clean_sort_spec},
+        $self->{_Clean_doctype},
+        $self->{_Clean_indexname},
         $size_limit,
         $clean_flag );
-    sql_delete_where_comma_flags ( $index, $table_name, $clean_flag );
+    sql_delete_where_comma_flags ( $dbh, $table_name, $clean_flag );
   }
   # and if we have any bcollection tables to clean, do them, too. it's
   # pretty kludgy to do this here rather than in a separate chunk of
@@ -117,30 +122,28 @@ sub clean {
   # that, to make sure). so our $table_name is the data table
   # name. (see, I told you it was kludgy)
   foreach my $bctn ( @{$self->{_Clean_bcollection_table_names}} ) {
-    sql_clear_all_comma_flags ( $index, $bctn );
-    my $sth = $dbh->prepare
-      ( $index->sql_clean_find_orphans ($bctn, $table_name) );
+    sql_clear_all_comma_flags ( $dbh, $bctn );
+    my $sth = $dbh->prepare ( sql_clean_find_orphans ($bctn, $table_name) );
     $sth->execute();
     while ( my $row = $sth->fetchrow_arrayref() ) {
       my $orphan_id = $row->[0];
       $dbh->do ( "UPDATE $bctn SET _comma_flag=$clean_flag WHERE doc_id="
                  . $dbh->quote($orphan_id) );
     }
-    sql_delete_where_comma_flags ( $index, $bctn, $clean_flag );
+    sql_delete_where_comma_flags ( $dbh, $bctn, $clean_flag );
   }
   # unset comma flag
-  sql_unset_table_comma_flag ( $index,$table_name );
+  sql_unset_table_comma_flag ( $dbh, $table_name );
   $self->{_Clean_in_progress} = 0;
 }
 
 sub DESTROY {
   my $self = shift();
-  if ( $self->{_Clean_in_progress} && $self->{_Clean_Index} ) {
+  if ( $self->{_Clean_in_progress} ) {
     # un-set table _comma flag
-    sql_unset_table_comma_flag( $self->{_Clean_Index},
+    sql_unset_table_comma_flag( $self->{_Clean_dbh},
                                 $self->{_Clean_table_name} );
   }
-  undef $self->{_Clean_Index};
 }
 
 1;
